@@ -7,6 +7,7 @@
 #include "dbutil.h"
 #include "util.h"
 #include "css.h"
+#include "wiki.h"
 
 #define ADD_ITEM_COUNT 5
 
@@ -26,6 +27,7 @@ void new_item_action();
 void new_item_submit_action();
 void delete_item_action();
 void delete_item_submit_action();
+void admin_help_action();
 void output_header(Project*, char*, char*, int);
 void output_footer();
 int cgiMain();
@@ -46,7 +48,8 @@ void register_actions()
     register_action_actions("new_item_submit", new_item_submit_action);
     register_action_actions("delete_item", delete_item_action);
     register_action_actions("delete_item_submit", delete_item_submit_action);
-    register_action_actions("default", menu_action);
+    register_action_actions("admin_help", admin_help_action);
+    register_action_actions("top", menu_action);
 }
 
 enum NAVI {
@@ -55,7 +58,8 @@ enum NAVI {
     NAVI_PROJECT,
     NAVI_ENV,
     NAVI_ITEM,
-    NAVI_STYLE
+    NAVI_STYLE,
+    NAVI_ADMIN_HELP
 };
 void output_header(Project* project, char* title, char* script_name, int navi)
 {
@@ -67,7 +71,7 @@ void output_header(Project* project, char* title, char* script_name, int navi)
             "\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />\n"
             "\t<meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\" />\n"
             "\t<meta http-equiv=\"Content-Style-type\" content=\"text/css\" />"
-            "\t<title>Starbug1 %s 管理ツール - %s</title>\n", project->name, title);
+            "\t<title>%s 管理ツール - %s</title>\n", project->name, title);
     o(      "\t<link rel=\"stylesheet\" type=\"text/css\" href=\"%s/../css/style.css\" />\n", cgiScriptName);
     o(      "\t<link rel=\"stylesheet\" type=\"text/css\" href=\"%s/../css/user.css\" />\n", cgiScriptName);
     if (script_name) {
@@ -77,7 +81,7 @@ void output_header(Project* project, char* title, char* script_name, int navi)
     o(      "</head>\n"
             "<body>\n"
             "<a name=\"top\"></a>\n"
-            "<h1 id=\"toptitle\" title=\"Starbug1 管理ツール\"><a href=\"http://sourceforge.jp/projects/starbug1/\"><img src=\"%s/../img/title.jpg\" alt=\"Starbug1\" /></a></h1>\n"
+            "<h1 id=\"toptitle\" title=\"管理ツール\"><a href=\"http://starbug1.sourceforge.jp/\"><img src=\"%s/../img/title.jpg\" alt=\"Starbug1\" /></a></h1>\n"
             "<ul id=\"menu\">\n", cgiScriptName);
     o(      "\t<li><a %s href=\"%s\">管理ツールメニュー</a></li>\n", navi == NAVI_MENU ? "class=\"current\"" : "", cgiScriptName);
     o(      "\t\t<li><a %s href=\"%s/project\">プロジェクトの設定</a></li>\n", navi == NAVI_PROJECT ? "class=\"current\"" : "", cgiScriptName);
@@ -85,6 +89,7 @@ void output_header(Project* project, char* title, char* script_name, int navi)
     o(      "\t\t<li><a %s href=\"%s/items\">項目設定</a></li>\n", navi == NAVI_ITEM ? "class=\"current\"" : "", cgiScriptName);
     o(      "\t\t<li><a %s href=\"%s/style\">スタイル設定</a></li>\n", navi == NAVI_STYLE ? "class=\"current\"" : "", cgiScriptName);
     o(      "\t<li><a href=\"%s/../db/starbug1.db\">バックアップ</a></li>\n", cgiScriptName);
+    o(      "\t\t<li><a %s href=\"%s/admin_help\">ヘルプ</a></li>\n", navi == NAVI_ADMIN_HELP ? "class=\"current\"" : "", cgiScriptName);
     o(      "\t<li><a href=\"%s/../index.cgi\">", cgiScriptName);h(project->name); o("トップへ</a></li>\n");
     o(      "</ul>\n"
             "<br clear=\"all\" />\n");
@@ -116,18 +121,20 @@ int cgiMain() {
  */
 void menu_action()
 {
-    Project* project;
+    Project* project_a = xalloc(sizeof(Project));
     char message[DEFAULT_LENGTH];
 
+    d("admin\n");
     db_init();
-    project = db_get_project();
-    output_header(project, "メニュー", NULL, NAVI_MENU);
+    project_a = db_get_project(project_a);
+    output_header(project_a, "メニュー", NULL, NAVI_MENU);
 
     cgiFormStringNoNewlines("message", message, DEFAULT_LENGTH);
     if (strlen(message) > 0) {
         o("<div class=\"complete_message\">"); h(message); o("&nbsp;</div>\n");
     }
-    o("<h2>%s 管理ツール</h2>", project->name);
+    o("<h2>%s 管理ツール</h2>", project_a->name);
+    xfree(project_a);
     o("<div id=\"admin_menu\">\n");
     o("\t<dl>\n");
     o("\t\t<dt><a href=\"%s/project\">プロジェクト設定</a></dt><dd>プロジェクトの基本的な情報の設定です。</dd>\n", cgiScriptName);
@@ -145,29 +152,26 @@ void menu_action()
  */
 void project_action()
 {
-    Project* project;
+    Project* project_a = xalloc(sizeof(Project));
 
     db_init();
-    project = db_get_project();
-    output_header(project, "プロジェクト設定", "management.js", NAVI_PROJECT);
+    project_a = db_get_project(project_a);
+    output_header(project_a, "プロジェクト設定", "management.js", NAVI_PROJECT);
 
-    o("<h2>%s 管理ツール</h2>", project->name);
+    o("<h2>%s 管理ツール</h2>", project_a->name);
     o("<div id=\"setting_form\">\n");
     o("\t<form id=\"management_form\" action=\"%s/project_submit\" method=\"post\">\n", cgiScriptName);
     o("\t\t<h3>プロジェクト設定</h3>\n");
     o("\t\t<table summary=\"project table\">\n");
     o("\t\t\t<tr>\n");
     o("\t\t\t\t<th>プロジェクト名</th>\n");
-    o("\t\t\t\t<td><input type=\"text\" name=\"project.name\" value=\"");h(project->name);o("\" maxlength=\"1023\" /></td>\n");
-    o("\t\t\t</tr>\n");
-    o("\t\t\t<tr>\n");
-    o("\t\t\t\t<th>説明</th>\n");
-    o("\t\t\t\t<td><input type=\"text\" name=\"project.description\" value=\"");h(project->description);o("\" maxlength=\"1023\" /></td>\n");
+    o("\t\t\t\t<td><input type=\"text\" name=\"project.name\" value=\"");h(project_a->name);o("\" maxlength=\"1023\" /></td>\n");
     o("\t\t\t</tr>\n");
     o("\t\t</table>\n");
     o("\t\t<input class=\"button\" type=\"submit\" value=\"更新\" />\n");
     o("\t</form>\n");
     o("</div>\n");
+    xfree(project_a);
     output_footer();
     db_finish();
 }
@@ -176,16 +180,15 @@ void project_action()
  */
 void project_submit_action()
 {
-    Project* project;
+    Project* project_a = xalloc(sizeof(Project));
 
     db_init();
     db_begin();
-    project = db_get_project();
-    cgiFormStringNoNewlines("project.name", project->name, DEFAULT_LENGTH);
-    cgiFormStringNoNewlines("project.description", project->description, DEFAULT_LENGTH);
-    db_update_project(project);
+    project_a = db_get_project(project_a);
+    cgiFormStringNoNewlines("project.name", project_a->name, DEFAULT_LENGTH);
+    db_update_project(project_a);
+    xfree(project_a);
 
-    project = db_get_project();
     db_commit();
     db_finish();
     redirect("", "更新しました");
@@ -195,13 +198,13 @@ void project_submit_action()
  */
 void env_action()
 {
-    Project* project;
+    Project* project_a = xalloc(sizeof(Project));
 
     db_init();
-    project = db_get_project();
-    output_header(project, "環境設定", "management.js", NAVI_ENV);
+    project_a = db_get_project(project_a);
+    output_header(project_a, "環境設定", "management.js", NAVI_ENV);
 
-    o("<h2>%s 管理ツール</h2>", project->name);
+    o("<h2>%s 管理ツール</h2>", project_a->name);
     o("<div id=\"setting_form\">\n");
     o("\t<form id=\"management_form\" action=\"%s/env_submit\" method=\"post\">\n", cgiScriptName);
     o("\t\t<h3>環境設定</h3>\n");
@@ -209,43 +212,16 @@ void env_action()
     o("\t\t\t<tr>\n");
     o("\t\t\t\t<th>home_url</th>\n");
     o("\t\t\t\t<td>\n");
-    o("\t\t\t\t\t<input type=\"text\" name=\"project.home_url\" value=\"");h(project->home_url);o("\" maxlength=\"1023\" />\n");
+    o("\t\t\t\t\t<input type=\"text\" name=\"project.home_url\" value=\"");h(project_a->home_url);o("\" maxlength=\"1023\" />\n");
     o("\t\t\t\t\t<div class=\"description\">各ページに表示されるナビゲータの、\"ホーム\" アンカーのリンク先を指定します。</div>\n");
     o("\t\t\t\t</td>\n");
     o("\t\t\t</tr>\n");
-    o("\t\t\t<tr>\n");
-    o("\t\t\t\t<th>smtp_server</th>\n");
-    o("\t\t\t\t<td>\n");
-    o("\t\t\t\t\t<input type=\"text\" name=\"project.smtp_server\" value=\"");h(project->smtp_server);o("\" maxlength=\"1023\" />\n");
-    o("\t\t\t\t\t<div class=\"description\">メール送信時に使用するSMTPサーバのホスト名です。</div>\n");
-    o("\t\t\t\t</td>\n");
-    o("\t\t\t</tr>\n");
-    o("\t\t\t<tr>\n");
-    o("\t\t\t\t<th>smtp_port</th>\n");
-    o("\t\t\t\t<td>\n");
-    o("\t\t\t\t\t<input type=\"text\" name=\"project.smtp_port\" value=\"%d\" maxlength=\"1023\" />", project->smtp_port);
-    o("\t\t\t\t\t<div class=\"description\">メール送信時に使用するSMTPサーバのポート番号です。</div>\n");
-    o("\t\t\t\t</td>\n");
-    o("\t\t\t</tr>\n");
-    o("\t\t\t<tr>\n");
-    o("\t\t\t\t<th>通知先アドレス</th>\n");
-    o("\t\t\t\t<td>\n");
-    o("\t\t\t\t\t<input type=\"text\" name=\"project.notify_address\" value=\"");h(project->notify_address);o("\" maxlength=\"1023\" />\n");
-    o("\t\t\t\t\t<div class=\"description\">投稿があったときに通知を行なうメールアドレスです。メーリングリストなどを指定してください。</div>\n");
-    o("\t\t\t\t</td>\n");
-    o("\t\t\t</tr>\n");
-    o("\t\t\t<tr>\n");
-    o("\t\t\t\t<th>管理者アドレス</th>\n");
-    o("\t\t\t\t<td>\n");
-    o("\t\t\t\t\t<input type=\"text\" name=\"project.admin_address\" value=\"");h(project->admin_address);o("\" maxlength=\"1023\" />\n");
-    o("\t\t\t\t\t<div class=\"description\">メール通知のfromに指定されるメールアドレスです。</div>\n");
-    o("\t\t\t\t</td>\n");
-    o("\t\t\t</tr>\n");
     o("\t\t</table>\n");
     o("\t\t<div class=\"description\">サーバ名、通知先アドレス、管理者アドレスが入力されていない場合、メール通知機能は機能しません。</div>\n");
     o("\t\t<input class=\"button\" type=\"submit\" value=\"更新\" />\n");
     o("\t</form>\n");
     o("</div>\n");
+    xfree(project_a);
     output_footer();
     db_finish();
 }
@@ -254,22 +230,16 @@ void env_action()
  */
 void env_submit_action()
 {
-    Project* project;
-    char smtp_port[DEFAULT_LENGTH];
+    Project* project_a = xalloc(sizeof(Project));
 
     db_init();
     db_begin();
-    project = db_get_project();
-    cgiFormStringNoNewlines("project.home_url", project->home_url, DEFAULT_LENGTH);
-    cgiFormStringNoNewlines("project.smtp_server", project->smtp_server, DEFAULT_LENGTH);
-    cgiFormStringNoNewlines("project.smtp_port", smtp_port, DEFAULT_LENGTH);
-    project->smtp_port = atoi(smtp_port);
-    cgiFormStringNoNewlines("project.notify_address", project->notify_address, DEFAULT_LENGTH);
-    cgiFormStringNoNewlines("project.admin_address", project->admin_address, DEFAULT_LENGTH);
+    project_a = db_get_project(project_a);
+    cgiFormStringNoNewlines("project.home_url", project_a->home_url, DEFAULT_LENGTH);
 
-    db_update_project(project);
+    db_update_project(project_a);
 
-    project = db_get_project();
+    xfree(project_a);
     db_commit();
     db_finish();
     redirect("", "更新しました");
@@ -279,15 +249,17 @@ void env_submit_action()
  */
 void items_action()
 {
-    Project* project;
+    Project* project_a = xalloc(sizeof(Project));
     List* element_types_a;
     Iterator* it;
 
     db_init();
-    project = db_get_project();
-    output_header(project, "項目設定", "management.js", NAVI_ITEM);
+    project_a = db_get_project(project_a);
+    output_header(project_a, "項目設定", "management.js", NAVI_ITEM);
 
-    o("<h2>%s 管理ツール</h2>", project->name);
+    o("<div id=\"top\">\n");
+    o("<h2>%s 管理ツール</h2>", project_a->name);
+    xfree(project_a);
     o("<div id=\"setting_form\">\n");
     o("\t<form id=\"management_form\" action=\"%s/items_submit\" method=\"post\">\n", cgiScriptName);
     o("\t\t<h3>項目設定</h3>\n");
@@ -309,7 +281,6 @@ void items_action()
         Iterator* it;
         list_alloc(items_a, ListItem);
         items_a = db_get_list_item(et->id, items_a);
-        o("\t\t<hr />\n");
         o("\t\t<a name=\"field%d\"></a>\n", et->id);
         o("\t\t<h4>"); h(et->name); o("</h4>\n");
         o("\t\t<div class=\"item_navigation\"><a href=\"#top\">このページのトップへ</a></div>\n");
@@ -475,15 +446,15 @@ void items_action()
         o("\t\t</table>\n");
         if (et->id > BASIC_ELEMENT_MAX) {
             /* 基本項目は削除できないようにする。 */
-            o("\t\t<div><a href=\"%s/delete_item/%d\">", cgiScriptName, et->id);o("この項目(");h(et->name);o(")の削除</a></div>\n");
+            o("\t\t<div class=\"delete_item\"><a href=\"%s/delete_item/%d\">", cgiScriptName, et->id);o("この項目(");h(et->name);o(")の削除</a></div>\n");
         }
         list_free(items_a);
+        o("\t\t<input class=\"button\" type=\"submit\" value=\"更新\" />\n");
     }
     list_free(element_types_a);
-    o("\t\t<hr />\n");
-    o("\t\t<input class=\"button\" type=\"submit\" value=\"更新\" />\n");
     o("\t</form>\n");
     o("</div>\n");
+    o("</div>\n");
 
     output_footer();
     db_finish();
@@ -493,11 +464,8 @@ void items_action()
  */
 void items_submit_action()
 {
-    Project* project;
-
     db_init();
     db_begin();
-    project = db_get_project();
     update_elements();
     db_commit();
     db_finish();
@@ -508,7 +476,6 @@ void update_elements()
 {
     char** ids;
     int i = -1;
-    ElementType* et = NULL;
     if ((cgiFormStringMultiple("field_ids", &ids)) == cgiFormNotFound) {
         die("cannot find field_ids.");
     }
@@ -518,72 +485,73 @@ void update_elements()
         char value[DEFAULT_LENGTH];
         List* items_a;
         Iterator* it;
+        ElementType* et_a = xalloc(sizeof(ElementType));
 
-        et = db_get_element_type(atoi(id));
+        et_a = db_get_element_type(atoi(id), et_a);
 
         d("while id: %s\n", id);
         d("while\n");
         sprintf(name, "field%s.name", id);
         cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-        strcpy(et->name, value);
+        strcpy(et_a->name, value);
 
         sprintf(name, "field%s.description", id);
         cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-        strcpy(et->description, value);
+        strcpy(et_a->description, value);
 
         sprintf(name, "field%s.ticket_property", id);
         cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-        et->ticket_property = atoi(value);
+        et_a->ticket_property = atoi(value);
 
         sprintf(name, "field%s.reply_property", id);
         cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-        et->reply_property = atoi(value);
+        et_a->reply_property = atoi(value);
 
         sprintf(name, "field%s.required", id);
         cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-        et->required = atoi(value);
+        et_a->required = atoi(value);
 
         sprintf(name, "field%s.display_in_list", id);
         cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-        et->display_in_list = atoi(value);
+        et_a->display_in_list = atoi(value);
 
         sprintf(name, "field%s.sort", id);
         cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-        et->sort = atoi(value);
+        et_a->sort = atoi(value);
 
         sprintf(name, "field%s.default_value", id);
         cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-        strcpy(et->default_value, value);
+        strcpy(et_a->default_value, value);
 
         sprintf(name, "field%s.auto_add_item", id);
         cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
         d("auto_add_item: %s\n", value);
-        et->auto_add_item = atoi(value);
-        d("auto_add_item: %d\n", et->auto_add_item);
+        et_a->auto_add_item = atoi(value);
+        d("auto_add_item: %d\n", et_a->auto_add_item);
 
-        db_update_element_type(et);
+        db_update_element_type(et_a);
 
         /* list_item */
-        switch (et->type) {
+        switch (et_a->type) {
             case ELEM_TYPE_LIST_SINGLE:
             case ELEM_TYPE_LIST_MULTI:
                 /* 選択要素のあるelementだけ、list_itemの更新を行なう。 */
                 list_alloc(items_a, ListItem);
-                items_a = db_get_list_item(et->id, items_a);
+                items_a = db_get_list_item(et_a->id, items_a);
                 foreach (it, items_a) {
                     ListItem* item = it->element;
                     strcpy(name, "");
                     strcpy(value, "");
-                    sprintf(name, "field%d.list_item%d.name", et->id, item->id);
+                    sprintf(name, "field%d.list_item%d.name", et_a->id, item->id);
                     cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
                     strcpy(item->name, value);
-                    sprintf(name, "field%d.list_item%d.close", et->id, item->id);
+                    sprintf(name, "field%d.list_item%d.close", et_a->id, item->id);
                     cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
                     item->close = atoi(value);
-                    sprintf(name, "field%d.list_item%d.sort", et->id, item->id);
+                    sprintf(name, "field%d.list_item%d.sort", et_a->id, item->id);
                     cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
                     item->sort = atoi(value);
-                    sprintf(name, "field%d.list_item%d.delete", et->id, item->id);
+                    sprintf(name, "field%d.list_item%d.delete", et_a->id, item->id);
                     cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
                     if (atoi(value) == 1)
                         db_delete_list_item(item->id);
@@ -593,37 +561,39 @@ void update_elements()
                 list_free(items_a);
                 strcpy(name, "");
                 strcpy(value, "");
-                sprintf(name, "field%d.list_item_new.name", et->id);
+                sprintf(name, "field%d.list_item_new.name", et_a->id);
                 cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
                 if (strlen(value) > 0) {
-                    ListItem* item = xalloc(sizeof(ListItem));
-                    item->element_type_id = et->id;
-                    strcpy(item->name, value);
-                    sprintf(name, "field%d.list_item_new.close", et->id);
+                    ListItem* item_a = xalloc(sizeof(ListItem));
+                    item_a->element_type_id = et_a->id;
+                    strcpy(item_a->name, value);
+                    sprintf(name, "field%d.list_item_new.close", et_a->id);
                     cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-                    item->close = atoi(value);
-                    sprintf(name, "field%d.list_item_new.sort", et->id);
+                    item_a->close = atoi(value);
+                    sprintf(name, "field%d.list_item_new.sort", et_a->id);
                     cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-                    item->sort = atoi(value);
-                    db_register_list_item(item);
+                    item_a->sort = atoi(value);
+                    db_register_list_item(item_a);
+                    xfree(item_a);
                 }
                 break;
         }
 
-        free(et);
+        xfree(et_a);
     }
     cgiStringArrayFree(ids);
 }
 void new_item_action()
 {
-    Project* project;
+    Project* project_a = xalloc(sizeof(Project));
     int i;
 
     db_init();
-    project = db_get_project();
-    output_header(project, "新規項目登録", "new_item.js", NAVI_OTHER);
+    project_a = db_get_project(project_a);
+    output_header(project_a, "新規項目登録", "new_item.js", NAVI_OTHER);
 
-    o("<h2>%s 管理ツール</h2>", project->name);
+    o("<h2>%s 管理ツール</h2>", project_a->name);
+    xfree(project_a);
     o(      "<div id=\"new_item\">\n"
             "<h3>項目の追加</h3>\n"
             "<div class=\"description\">チケットに新しい項目を追加します。追加する項目についての情報を入力し、追加ボタンを押してください。</div>\n"
@@ -755,56 +725,56 @@ void new_item_action()
 }
 void new_item_submit_action()
 {
-    ElementType* et = xalloc(sizeof(ElementType));
-    Project* project;
+    ElementType* et_a = xalloc(sizeof(ElementType));
     char value[DEFAULT_LENGTH];
     int i, e_type_id;
 
     db_init();
     db_begin();
-    project = db_get_project();
 
-    cgiFormStringNoNewlines("field.name", et->name, DEFAULT_LENGTH);
+    cgiFormStringNoNewlines("field.name", et_a->name, DEFAULT_LENGTH);
 
-    cgiFormStringNoNewlines("field.description", et->description, DEFAULT_LENGTH);
+    cgiFormStringNoNewlines("field.description", et_a->description, DEFAULT_LENGTH);
 
     cgiFormStringNoNewlines("field.type", value, DEFAULT_LENGTH);
-    et->type = atoi(value);
+    et_a->type = atoi(value);
 
     cgiFormStringNoNewlines("field.ticket_property", value, DEFAULT_LENGTH);
-    et->ticket_property = atoi(value);
+    et_a->ticket_property = atoi(value);
 
     cgiFormStringNoNewlines("field.reply_property", value, DEFAULT_LENGTH);
-    et->reply_property = atoi(value);
+    et_a->reply_property = atoi(value);
 
     cgiFormStringNoNewlines("field.display_in_list", value, DEFAULT_LENGTH);
-    et->display_in_list = atoi(value);
+    et_a->display_in_list = atoi(value);
     cgiFormStringNoNewlines("field.auto_add_item", value, DEFAULT_LENGTH);
-    et->auto_add_item = atoi(value);
+    et_a->auto_add_item = atoi(value);
     cgiFormStringNoNewlines("field.sort", value, DEFAULT_LENGTH);
-    et->sort = atoi(value);
-    e_type_id = db_register_element_type(et);
-    switch (et->type) {
+    et_a->sort = atoi(value);
+    e_type_id = db_register_element_type(et_a);
+    switch (et_a->type) {
         case ELEM_TYPE_LIST_SINGLE:
         case ELEM_TYPE_LIST_MULTI:
             for (i = 0; i < ADD_ITEM_COUNT; i++) {
                 char name[DEFAULT_LENGTH];
-                ListItem* item = xalloc(sizeof(ListItem));
-                item->element_type_id = e_type_id;
+                ListItem* item_a = xalloc(sizeof(ListItem));
+                item_a->element_type_id = e_type_id;
                 sprintf(name, "field.list_item_new%d.name", i);
-                cgiFormStringNoNewlines(name, item->name, DEFAULT_LENGTH);
-                if (strlen(item->name) > 0) {
+                cgiFormStringNoNewlines(name, item_a->name, DEFAULT_LENGTH);
+                if (strlen(item_a->name) > 0) {
                     sprintf(name, "field.list_item_new%d.close", i);
                     cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-                    item->close = atoi(value);
+                    item_a->close = atoi(value);
                     sprintf(name, "field.list_item_new%d.sort", i);
                     cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-                    item->sort = atoi(value);
-                    db_register_list_item(item);
+                    item_a->sort = atoi(value);
+                    db_register_list_item(item_a);
                 }
+                xfree(item_a);
             }
             break;
     }
+    xfree(et_a);
     db_commit();
     db_finish();
     redirect("", "追加しました");
@@ -814,21 +784,22 @@ void delete_item_action()
     char path_info[DEFAULT_LENGTH];
     char* e_type_id;
     int iid;
-    Project* project;
-    ElementType* et;
+    Project* project_a = xalloc(sizeof(Project));
+    ElementType* et_a = xalloc(sizeof(ElementType));
 
     strcpy(path_info, cgiPathInfo);
     e_type_id = strchr(path_info + 1, '/');
     if (e_type_id) e_type_id++;
     iid = atoi(e_type_id);
     db_init();
-    project = db_get_project();
-    output_header(project, "項目削除", "delete_item.js", NAVI_OTHER);
+    project_a = db_get_project(project_a);
+    output_header(project_a, "項目削除", "delete_item.js", NAVI_OTHER);
 
-    et = db_get_element_type(iid);
-    o("<h2>%s 管理ツール</h2>", project->name);
+    et_a = db_get_element_type(iid, et_a);
+    o("<h2>%s 管理ツール</h2>", project_a->name);
+    xfree(project_a);
     o(      "<div id=\"delete_item/%d\">\n", iid);
-    o(      "<h3>項目(");h(et->name);o(")の削除</h3>\n"
+    o(      "<h3>項目(");h(et_a->name);o(")の削除</h3>\n"
             "<form id=\"delete_item_form\" action=\"%s/delete_item_submit/%d\" method=\"post\">\n"
             "<div class=\"infomation\"><strong>削除すると元には戻せません。"
             "登録されているチケットの項目についても参照できなくなります。</strong></div>"
@@ -838,6 +809,7 @@ void delete_item_action()
     o(      "</div>\n");
     db_finish();
     output_footer();
+    xfree(et_a);
 }
 void delete_item_submit_action()
 {
@@ -860,13 +832,13 @@ void delete_item_submit_action()
 }
 void style_action()
 {
-    Project* project;
+    Project* project_a = xalloc(sizeof(Project));
     db_init();
-    project = db_get_project();
-    output_header(project, "スタイル設定", "style.js", NAVI_STYLE);
-    db_finish();
-    o(      "<h2>スタイル編集</h2>\n"
-            "<div id=\"top\">\n"
+    project_a = db_get_project(project_a);
+    output_header(project_a, "スタイル設定", "style.js", NAVI_STYLE);
+    o(      "<h2>%s 管理ツール</h2>", project_a->name);
+    xfree(project_a);
+    o(      "<div id=\"top\">\n"
             "<h3>スタイルシートの編集</h3>\n"
             "<div id=\"description\">スタイルシートの編集を行ない、更新ボタンを押してください。</div>\n"
             "<form id=\"edit_css_form\" action=\"%s/style_submit\" method=\"post\">\n", cgiScriptName);
@@ -876,7 +848,39 @@ void style_action()
             "<div>&nbsp;</div>\n"
             "<input class=\"button\" type=\"submit\" value=\"更新\" />\n"
             "</form>");
+    {
+        List* element_types_a;
+        Iterator* it;
+        o(      "<div class=\"description\">以下は、一覧の背景設定用のサンプルです。</div>\n");
+        o(      "<pre>\n");
+        list_alloc(element_types_a, ElementType);
+        element_types_a = db_get_element_types_all(element_types_a);
+        foreach (it, element_types_a) {
+            ElementType* et = it->element;
+            if (et->type == ELEM_TYPE_LIST_SINGLE && et->display_in_list) {
+                List* items_a;
+                Iterator* it_item;
+                list_alloc(items_a, ListItem);
+                items_a = db_get_list_item(et->id, items_a);
+                o(  "/* ================================ */ \n"
+                    "/* チケット一覧の"); h(et->name); o("の背景色設定     */\n");
+                o(  "/* ================================ */ \n");
+                foreach (it_item, items_a) {
+                    ListItem* item = it_item->element;
+                    o("/* "); h(item->name); o(" */\n");
+                    o("#ticket_list td.field%d-", et->id);
+                    css_field(item->name);
+                    o(" {\n");
+                    o(" background-color: lightyellow !important;\n");
+                    o("}\n");
+                }
+            }
+        }
+        list_free(element_types_a);
+        o(      "</pre>\n");
+    }
     o(      "</div>\n");
+    db_finish();
     output_footer();
 }
 void style_submit_action()
@@ -887,6 +891,20 @@ void style_submit_action()
     css_save("css/user.css", value_a);
 
     redirect("", "更新しました。");
-    free(value_a);
+    xfree(value_a);
+}
+void admin_help_action()
+{
+    Project* project_a = xalloc(sizeof(Project));
+    db_init();
+    project_a = db_get_project(project_a);
+    output_header(project_a, "ヘルプ", NULL, NAVI_ADMIN_HELP);
+    o(      "<h2>");h(project_a->name);o("</h2>\n"
+            "<div id=\"top\">\n");
+    wiki_out("adminhelp");
+    o(      "</div>\n");
+    xfree(project_a);
+    db_finish();
+    output_footer();
 }
 /* vim: set ts=4 sw=4 sts=4 expandtab: */
