@@ -66,9 +66,8 @@ List* db_get_element_types_all(List* elements)
 {
     return db_get_element_types(1, elements);
 }
-ElementType* db_get_element_type(int id)
+ElementType* db_get_element_type(int id, ElementType* e)
 {
-    ElementType* e;
     int r;
     const char *sql;
     sqlite3_stmt *stmt = NULL;
@@ -81,10 +80,8 @@ ElementType* db_get_element_type(int id)
     sqlite3_reset(stmt);
     sqlite3_bind_int(stmt, 1, id);
 
-    e = NULL;
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
         const unsigned char* value;
-        e = (ElementType*)xalloc(sizeof(ElementType));
         e->id = sqlite3_column_int(stmt, 0);
         e->type = sqlite3_column_int(stmt, 1);
         e->ticket_property = sqlite3_column_int(stmt, 2);
@@ -168,7 +165,7 @@ int db_register_ticket(Message* ticket)
     List* elements;
     Iterator* it;
     char sql[DEFAULT_LENGTH];
-    int i, loop = 0, closed = 0;
+    int i, closed = 0;
     sqlite3_stmt *stmt = NULL;
     int message_id;
     int register_mode = ticket->id == -1;
@@ -217,10 +214,7 @@ int db_register_ticket(Message* ticket)
         Element* e = it->element;
         sqlite3_bind_text(stmt, i++, e->str_val, strlen(e->str_val), NULL);
     }
-    while (SQLITE_DONE != sqlite3_step(stmt)){
-        if (loop++ > 1000)
-            goto error;
-    }
+    if (exec_and_wait_4_done(stmt) != SQLITE_RETURN_OK) goto error;
     message_id = sqlite3_last_insert_rowid(db);
     sqlite3_finalize(stmt);
     /* message_id を更新する。 */
@@ -266,8 +260,8 @@ int db_register_ticket(Message* ticket)
                     COLUMN_TYPE_BLOB, content_a,
                     COLUMN_TYPE_END) == 0)
                 die("insert failed.");
-            free(content_a->blob);
-            free(content_a);
+            xfree(content_a->blob);
+            xfree(content_a);
         }
     }
     return ticket->id;
@@ -308,6 +302,7 @@ char* get_search_sql_string(List* conditions, Condition* sort, char* q, char* sq
         foreach (it, conditions) {
             Condition* cond = it->element;
             char val[DEFAULT_LENGTH];
+            if (cond->element_type_id < 0) continue;
             if (i++) strcat(sql_string, " and ");
             switch (cond->condition_type) {
                 case CONDITION_TYPE_DATE_FROM:
@@ -324,6 +319,30 @@ char* get_search_sql_string(List* conditions, Condition* sort, char* q, char* sq
             }
             strcat(sql_string, val);
         }
+        foreach (it, conditions) {
+            Condition* cond = it->element;
+            char name[DEFAULT_LENGTH];
+            char val[DEFAULT_LENGTH];
+            if (cond->element_type_id > 0) continue;
+            if (i++) strcat(sql_string, " and ");
+            switch (cond->element_type_id) {
+                case ELEM_ID_REGISTERDATE:
+                    sprintf(name, "org_m.registerdate");
+                    break;
+                case ELEM_ID_LASTREGISTERDATE:
+                    sprintf(name, "m.registerdate");
+                    break;
+            }
+            switch (cond->condition_type) {
+                case CONDITION_TYPE_DATE_FROM:
+                    sprintf(val, " (%s >= ?) ", name);
+                    break;
+                case CONDITION_TYPE_DATE_TO:
+                    sprintf(val, " (%s <= ?) ", name);
+                    break;
+            }
+            strcat(sql_string, val);
+        }
     }
     if (strlen(q)) {
         char columns[DEFAULT_LENGTH];
@@ -341,7 +360,7 @@ char* get_search_sql_string(List* conditions, Condition* sort, char* q, char* sq
 
     strcat(sql_string, " group by t.id ");
     strcat(sql_string, " order by ");
-    if (sort) {
+    if (sort != NULL && sort->element_type_id != 0) {
         char column[DEFAULT_LENGTH];
         char sort_type[DEFAULT_LENGTH];
         sprintf(sort_type, "%s", strstr(sort->value, "reverse") ? "desc" : "asc");
@@ -389,7 +408,7 @@ int set_conditions(sqlite3_stmt* stmt, List* conditions, char* q)
     }
     return n;
 }
-SearchResult* db_get_tickets_by_status(char* status, List* messages)
+SearchResult* db_get_tickets_by_status(char* status, List* messages, SearchResult* result)
 {
     int r, n, hit_count = 0;
     char buffer[VALUE_LENGTH];
@@ -397,7 +416,6 @@ SearchResult* db_get_tickets_by_status(char* status, List* messages)
     List* conditions;
     Condition* cond_status;
     sqlite3_stmt *stmt = NULL;
-    SearchResult* result = (SearchResult*)xalloc(sizeof(SearchResult));
 
     list_alloc(conditions, Condition);
     cond_status = list_new_element(conditions);
@@ -412,7 +430,7 @@ SearchResult* db_get_tickets_by_status(char* status, List* messages)
     if (sqlite3_prepare(db, sql, strlen(sql), &stmt, NULL) == SQLITE_ERROR) goto error;
     sqlite3_reset(stmt);
     n = set_conditions(stmt, conditions, "");
-    sqlite3_bind_int(stmt, n++, LIST_PER_PAGE);
+    sqlite3_bind_int(stmt, n++, LIST_COUNT_PER_LIST_PAGE);
 
     /* hitした分のticket_idを取得する。 */
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
@@ -431,21 +449,20 @@ SearchResult* db_get_tickets_by_status(char* status, List* messages)
     return result;
 ERROR_LABEL
 }
-SearchResult* db_search_tickets(List* conditions, char* q, Condition* sorts, int page, List* messages)
+SearchResult* db_search_tickets(List* conditions, char* q, Condition* sorts, int page, List* messages, SearchResult* result)
 {
     int r, n;
     char buffer[VALUE_LENGTH];
     char* sql = get_search_sql_string(conditions, sorts, q, buffer);
     sqlite3_stmt *stmt = NULL;
-    SearchResult* result = (SearchResult*)xalloc(sizeof(SearchResult));
     result->messages = messages;
 
     strcat(sql, " limit ? offset ? ");
     if (sqlite3_prepare(db, sql, strlen(sql), &stmt, NULL) == SQLITE_ERROR) goto error;
     sqlite3_reset(stmt);
     n = set_conditions(stmt, conditions, q);
-    sqlite3_bind_int(stmt, n++, LIST_PER_PAGE);
-    sqlite3_bind_int(stmt, n++, page * LIST_PER_PAGE);
+    sqlite3_bind_int(stmt, n++, LIST_COUNT_PER_SEARCH_PAGE);
+    sqlite3_bind_int(stmt, n++, page * LIST_COUNT_PER_SEARCH_PAGE);
 
     /* 1ページ分のticket_idを取得する。 */
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
@@ -480,6 +497,33 @@ SearchResult* db_search_tickets(List* conditions, char* q, Condition* sorts, int
     return result;
 ERROR_LABEL
 }
+SearchResult* db_search_tickets_4_report(List* conditions, char* q, Condition* sorts, List* messages, SearchResult* result)
+{
+    int r, n;
+    char buffer[VALUE_LENGTH];
+    char* sql = get_search_sql_string(conditions, sorts, q, buffer);
+    sqlite3_stmt *stmt = NULL;
+    result->messages = messages;
+
+    if (sqlite3_prepare(db, sql, strlen(sql), &stmt, NULL) == SQLITE_ERROR) goto error;
+    sqlite3_reset(stmt);
+    n = set_conditions(stmt, conditions, q);
+
+    /* ticket_idを取得する。 */
+    while (SQLITE_ROW == (r = sqlite3_step(stmt))){
+        Message* message = list_new_element(result->messages);
+        message->id = sqlite3_column_int(stmt, 0);
+        list_add(result->messages, message);
+        result->hit_count++;
+    }
+    if (SQLITE_DONE != r)
+        goto error;
+
+    sqlite3_finalize(stmt);
+
+    return result;
+ERROR_LABEL
+}
 void create_columns_exp(List* element_types, char* table_name, char* buf)
 {
     Iterator* it;
@@ -493,7 +537,7 @@ void create_columns_exp(List* element_types, char* table_name, char* buf)
 static void set_str_val(Element* e, const unsigned char* str_val)
 {
     if (str_val != NULL) {
-        e->str_val = (char*)xalloc(sizeof(char) * strlen(str_val) + 1);
+        e->str_val = xalloc(sizeof(char) * strlen(str_val) + 1);
         strcpy(e->str_val, str_val);
     }
 }
@@ -510,7 +554,7 @@ List* db_get_last_elements_4_list(int ticket_id, List* elements)
     list_alloc(element_types_a, ElementType);
     element_types_a = db_get_element_types_4_list(element_types_a);
     create_columns_exp(element_types_a, "last_m", columns);
-    sprintf(sql, "select t.id, org_m.field%d, l.id ", ELEM_ID_SENDER);
+    sprintf(sql, "select t.id, org_m.field%d ", ELEM_ID_SENDER);
     strcat(sql, columns);
     sprintf(sql_suf, 
             "  , t.registerdate, last_m.registerdate,  "
@@ -518,8 +562,7 @@ List* db_get_last_elements_4_list(int ticket_id, List* elements)
             "from ticket as t "
             "inner join message as last_m on last_m.id = t.last_message_id "
             "inner join message as org_m on org_m.id = t.original_message_id "
-            "left join list_item as l on l.element_type_id = %d and l.name = last_m.field%d "
-            "where t.id = ?", ELEM_ID_STATUS, ELEM_ID_STATUS);
+            "where t.id = ?");
     strcat(sql, sql_suf);
     if (sqlite3_prepare(db, sql, strlen(sql), &stmt, NULL) == SQLITE_ERROR) goto error;
     sqlite3_reset(stmt);
@@ -527,7 +570,6 @@ List* db_get_last_elements_4_list(int ticket_id, List* elements)
 
     while (SQLITE_ROW == (r = sqlite3_step(stmt))) {
         int i = 0;
-        const unsigned char* status_id;
         Element* e;
         /* ID */
         e = list_new_element(elements);
@@ -539,15 +581,11 @@ List* db_get_last_elements_4_list(int ticket_id, List* elements)
         e->element_type_id = ELEM_ID_ORG_SENDER;
         set_str_val(e, sqlite3_column_text(stmt, i++));
         list_add(elements, e);
-        /* status item_list id */
-        status_id = sqlite3_column_text(stmt, i++);
         foreach (it, element_types_a) {
             ElementType* et = it->element;
             e = list_new_element(elements);
             e->element_type_id = et->id;
             set_str_val(e, sqlite3_column_text(stmt, i++));
-            if (et->id == ELEM_ID_STATUS && status_id != 0)
-                e->list_item_id = atoi(status_id); /* 状態のスタイルシートのために、list_item.idを設定 */
             list_add(elements, e);
         }
         /* 投稿日時 */
@@ -588,37 +626,48 @@ List* db_get_last_elements(int ticket_id, List* elements)
     create_columns_exp(element_types_a, "m", columns);
     strcpy(sql, "select m.id");
     strcat(sql, columns);
+    strcat(sql, 
+            "  , t.registerdate, last_m.registerdate,  "
+            "  julianday(current_date) - julianday(date(last_m.registerdate)) as passed_date ");
     strcat(sql,
             "from ticket as t "
+            "inner join message as last_m on last_m.id = t.last_message_id "
             "inner join message as m on m.id = t.last_message_id "
-            "where ticket_id = ?");
+            "where t.id = ?");
     if (sqlite3_prepare(db, sql, strlen(sql), &stmt, NULL) == SQLITE_ERROR) goto error;
     sqlite3_reset(stmt);
     sqlite3_bind_int(stmt, 1, ticket_id);
 
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
-        const unsigned char* str_val;
         int i = 0;
         Iterator* it;
+        Element* e;
         Element* e_id = list_new_element(elements);
         e_id->element_type_id = ELEM_ID_ID;
-        str_val = sqlite3_column_text(stmt, i++);
-        if (str_val != NULL) {
-            e_id->str_val = (char*)xalloc(sizeof(char) * strlen(str_val) + 1);
-            strcpy(e_id->str_val, str_val);
-        }
+        set_str_val(e_id, sqlite3_column_text(stmt, i++));
         list_add(elements, e_id);
         foreach (it, element_types_a) {
             ElementType* et = it->element;
-            Element* e = list_new_element(elements);
+            e = list_new_element(elements);
             e->element_type_id = et->id;
-            str_val = sqlite3_column_text(stmt, i++);
-            if (str_val != NULL) {
-                e->str_val = (char*)xalloc(sizeof(char) * strlen(str_val) + 1);
-                strcpy(e->str_val, str_val);
-            }
+            set_str_val(e, sqlite3_column_text(stmt, i++));
             list_add(elements, e);
         }
+        /* 投稿日時 */
+        e = list_new_element(elements);
+        e->element_type_id = ELEM_ID_REGISTERDATE;
+        set_str_val(e, sqlite3_column_text(stmt, i++));
+        list_add(elements, e);
+        /* 最終更新日時 */
+        e = list_new_element(elements);
+        e->element_type_id = ELEM_ID_LASTREGISTERDATE;
+        set_str_val(e, sqlite3_column_text(stmt, i++));
+        list_add(elements, e);
+        /* 最終更新日時からの経過日数 */
+        e = list_new_element(elements);
+        e->element_type_id = ELEM_ID_LASTREGISTERDATE_PASSED;
+        set_str_val(e, sqlite3_column_text(stmt, i++));
+        list_add(elements, e);
     }
     if (SQLITE_DONE != r)
         goto error;
@@ -652,25 +701,16 @@ List* db_get_elements(int message_id, List* elements)
 
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
         Iterator* it;
-        const unsigned char* str_val;
         int i = 0;
         Element* e = list_new_element(elements);
         e->element_type_id = ELEM_ID_LASTREGISTERDATE;
-        str_val = sqlite3_column_text(stmt, i++);
-        if (str_val != NULL) {
-            e->str_val = (char*)xalloc(sizeof(char) * strlen(str_val) + 1);
-            strcpy(e->str_val, str_val);
-        }
+        set_str_val(e, sqlite3_column_text(stmt, i++));
         list_add(elements, e);
         foreach (it, element_types_a) {
             ElementType* et = it->element;
             e = list_new_element(elements);
             e->element_type_id = et->id;
-            str_val = sqlite3_column_text(stmt, i++);
-            if (str_val != NULL) {
-                e->str_val = (char*)xalloc(sizeof(char) * strlen(str_val) + 1);
-                strcpy(e->str_val, str_val);
-            }
+            set_str_val(e, sqlite3_column_text(stmt, i++));
             list_add(elements, e);
         }
     }
@@ -684,7 +724,7 @@ List* db_get_elements(int message_id, List* elements)
 
 ERROR_LABEL
 }
-int* db_get_message_ids(int ticket_id)
+int* db_get_message_ids_a(int ticket_id)
 {
     int* message_ids = NULL;
     int r, i = 0;
@@ -698,7 +738,7 @@ int* db_get_message_ids(int ticket_id)
     if (SQLITE_ROW != (r = sqlite3_step(stmt))) {
         goto error;
     }
-    message_ids = (int*)xalloc(sizeof(int) * (sqlite3_column_int(stmt, 0) + 1));
+    message_ids = xalloc(sizeof(int) * (sqlite3_column_int(stmt, 0) + 1));
     sqlite3_finalize(stmt);
 
     sql = "select id from message as m where m.ticket_id = ? order by m.registerdate";
@@ -719,26 +759,19 @@ int* db_get_message_ids(int ticket_id)
 ERROR_LABEL
 }
 
-Project* db_get_project()
+Project* db_get_project(Project* project)
 {
-    Project* project = NULL;
     int r;
     const char *sql;
     sqlite3_stmt *stmt = NULL;
 
-    sql = "select name, description, home_url, smtp_server, smtp_port, notify_address, admin_address from project";
+    sql = "select name, home_url from project";
     if (sqlite3_prepare(db, sql, strlen(sql), &stmt, NULL) == SQLITE_ERROR) goto error;
     sqlite3_reset(stmt);
 
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
-        project = (Project*)xalloc(sizeof(Project));
         strcpy(project->name, sqlite3_column_text(stmt, 0));
-        strcpy(project->description, sqlite3_column_text(stmt, 1));
-        strcpy(project->home_url, sqlite3_column_text(stmt, 2));
-        strcpy(project->smtp_server, sqlite3_column_text(stmt, 3));
-        project->smtp_port = sqlite3_column_int(stmt, 4);
-        strcpy(project->notify_address, sqlite3_column_text(stmt, 5));
-        strcpy(project->admin_address, sqlite3_column_text(stmt, 6));
+        strcpy(project->home_url, sqlite3_column_text(stmt, 1));
         break;
     }
 
@@ -751,15 +784,9 @@ void db_update_project(Project* project)
 {
     if (exec_query(
             "update project set "
-            "name = ?, description = ?, home_url = ?, "
-            "smtp_server = ?, smtp_port = ?, notify_address = ?, admin_address = ?",
+            "name = ?, home_url = ? ",
             COLUMN_TYPE_TEXT, project->name,
-            COLUMN_TYPE_TEXT, project->description,
             COLUMN_TYPE_TEXT, project->home_url,
-            COLUMN_TYPE_TEXT, project->smtp_server,
-            COLUMN_TYPE_INT, project->smtp_port,
-            COLUMN_TYPE_TEXT, project->notify_address,
-            COLUMN_TYPE_TEXT, project->admin_address,
             COLUMN_TYPE_END) != 1)
         die("no project to update? or too many?");
 }
@@ -945,7 +972,7 @@ List* db_get_statictics_multi(List* states, int element_type_id)
     sqlite3_stmt *stmt = NULL;
 
     sprintf(sql, 
-            "select l.name as name, count(t.id) as count "
+            "select l.id, l.name as name, count(t.id) as count "
             "from ticket as t "
             "inner join message as m "
             " on m.id = t.last_message_id "
@@ -958,8 +985,9 @@ List* db_get_statictics_multi(List* states, int element_type_id)
 
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
         State* s = list_new_element(states);
-        strcpy(s->name, sqlite3_column_text(stmt, 0));
-        s->count = sqlite3_column_int(stmt, 1);
+        s->id = sqlite3_column_int(stmt, 0);
+        strcpy(s->name, sqlite3_column_text(stmt, 1));
+        s->count = sqlite3_column_int(stmt, 2);
         list_add(states, s);
     }
     if (SQLITE_DONE != r)
@@ -978,7 +1006,7 @@ List* db_get_statictics(List* states, int element_type_id)
     sqlite3_stmt *stmt = NULL;
 
     sprintf(sql, 
-            "select l.name as name, count(t.id) as count "
+            "select l.id, l.name as name, count(t.id) as count "
             "from ticket as t "
             "inner join message as m "
             " on m.id = t.last_message_id "
@@ -991,8 +1019,9 @@ List* db_get_statictics(List* states, int element_type_id)
 
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
         State* s = list_new_element(states);
-        strcpy(s->name, sqlite3_column_text(stmt, 0));
-        s->count = sqlite3_column_int(stmt, 1);
+        s->id = sqlite3_column_int(stmt, 0);
+        strcpy(s->name, sqlite3_column_text(stmt, 1));
+        s->count = sqlite3_column_int(stmt, 2);
         list_add(states, s);
     }
     if (SQLITE_DONE != r)
@@ -1004,9 +1033,8 @@ List* db_get_statictics(List* states, int element_type_id)
 
 ERROR_LABEL
 }
-ElementFile* db_get_element_file(int id)
+ElementFile* db_get_element_file(int id, ElementFile* file)
 {
-    ElementFile* file;
     int r;
     const char *sql;
     sqlite3_stmt *stmt = NULL;
@@ -1018,19 +1046,17 @@ ElementFile* db_get_element_file(int id)
     sqlite3_reset(stmt);
     sqlite3_bind_int(stmt, 1, id);
 
-    file = NULL;
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
         int len;
         char* p_src;
         char* p_dist;
-        file = (ElementFile*)xalloc(sizeof(ElementFile));
         file->id = sqlite3_column_int(stmt, 0);
         file->element_type_id = sqlite3_column_int(stmt, 1);
         strcpy(file->name, sqlite3_column_text(stmt, 2));
         file->size = sqlite3_column_int(stmt, 3);
         strcpy(file->mime_type, sqlite3_column_text(stmt, 4));
         len = sqlite3_column_bytes(stmt, 5);
-        p_dist = file->blob = (char*)xalloc(sizeof(char) * len);
+        p_dist = file->blob = xalloc(sizeof(char) * len);
         p_src = (char*)sqlite3_column_blob(stmt, 5);
         while (len--) {
             *p_dist = *p_src;
@@ -1082,4 +1108,48 @@ int db_get_element_file_id(int message_id, int element_type_id)
             COLUMN_TYPE_INT, element_type_id,
             COLUMN_TYPE_END);
 }
+void db_register_wiki(Wiki* wiki)
+{
+    char registerdate[20];
+    set_date_string(registerdate);
+    exec_query(
+            "insert into wiki (id, name, content, registerdate)"
+            "values (NULL, ?, ?, ?)",
+            COLUMN_TYPE_TEXT, wiki->name,
+            COLUMN_TYPE_TEXT, wiki->content,
+            COLUMN_TYPE_TEXT, registerdate,
+            COLUMN_TYPE_END);
+}
+Wiki* db_get_newest_wiki(char* page_name, Wiki* wiki)
+{
+    int r;
+    char sql[DEFAULT_LENGTH];
+    sqlite3_stmt *stmt = NULL;
+    
+    sprintf(sql, 
+            "select w.id, w.name, w.content "
+            "from wiki as w "
+            "where name = ? "
+            "order by w.registerdate desc "
+            "limit 1 ");
+
+    if (sqlite3_prepare(db, sql, strlen(sql), &stmt, NULL) == SQLITE_ERROR) goto error;
+    sqlite3_reset(stmt);
+    sqlite3_bind_text(stmt, 1, page_name, strlen(page_name), NULL);
+
+    while (SQLITE_ROW == (r = sqlite3_step(stmt))) {
+        const unsigned char* value;
+        wiki->id = sqlite3_column_int(stmt, 0);
+        strcpy(wiki->name, sqlite3_column_text(stmt, 1));
+        value = sqlite3_column_text(stmt, 2);
+        wiki->content = xalloc(sizeof(char) * strlen(value) + 1);
+        strcpy(wiki->content, value);
+        break;
+    }
+
+    sqlite3_finalize(stmt);
+    return wiki;
+
+ERROR_LABEL
+}
 /* vim: set ts=4 sw=4 sts=4 expandtab: */
