@@ -2,6 +2,7 @@ Event.observe(window, 'load', initPage);
 
 function initPage(e) {
     var required_fields = new Array();
+    var date_fields = new Array();
     var status_original = $F('field3');
     Event.observe('reply_form', 'submit', 
         function(e) {
@@ -12,8 +13,9 @@ function initPage(e) {
                         return false;
                     }
                 }
-                if (!check_input_value(required_fields)) {
-                    alert('入力されていない必須項目があります。確認してください。');
+                if (!check_input_value(required_fields) ||
+                        !validate_datefield(date_fields)) {
+                    alert('入力されていない必須項目または不正な値が指定された項目があります。確認してください。');
                     Event.stop(e);
                     return false;
                 }
@@ -27,30 +29,6 @@ function initPage(e) {
             }
         }
     );
-    var i = 1;
-    while (true) {
-        var field = $('field' + i++);
-        if (field == undefined) break;
-        var required = $(field.id + '.required');
-        if (required == undefined) continue;
-        required_fields.push(field);
-    }
-}
-function check_input_value(fields) {
-    var ret = $A(fields).findAll(
-        function(f) {
-            var empty = $F(f).empty();
-            var elem = $(f.id + '.required');
-            if (empty) {
-                elem.innerHTML = "必須項目です。入力してください。";
-                elem.style.display = "block";
-                f.focus();
-            } else {
-                elem.innerHTML = "";
-                elem.style.display = "none";
-            }
-            return empty;
-        }
-    );
-    return (ret.length == 0);
+    register_required_fields(required_fields, required_field_indexs);
+    register_date_fields(date_fields, date_field_indexs);
 }
