@@ -29,6 +29,9 @@ unsigned long url_encode(unsigned char*, unsigned char*, unsigned long);
 void redirect(char*, char*);
 void free_element_list(List*);
 void* xalloc(size_t);
+void xfree(void*);
+void csv_field(char*);
+void css_field(char*);
 
 #endif
 /* vim: set ts=4 sw=4 sts=4 expandtab: */
